package ruben.uoc.comedorescolar.activities;

import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.model.Menu;
import ruben.uoc.comedorescolar.model.UsuarioRol;

public class CalendarioActivity extends AppCompatActivity implements View.OnClickListener{

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
    private Date diaActual, primerDia, ultimoDia;
    private String keyCentro;

    private TextView dia1, dia2, dia3, dia4, dia5;
    private TextView estadoDia1, estadoDia2, estadoDia3, estadoDia4, estadoDia5;
    private TextView estadoColor1, estadoColor2, estadoColor3, estadoColor4, estadoColor5;

    private UsuarioRol usuarioRol;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_calendario);

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();

        // Botón para conseguir los días anteriores
        ImageView imagenAnteriores = findViewById(R.id.image_anterior);
        imagenAnteriores.setOnClickListener(this);

        // Botón para conseguir los días posteriores
        ImageView imagenPosteiores = findViewById(R.id.image_siguiente);
        imagenPosteiores.setOnClickListener(this);

        cargarElementosVista();

        diaActual = new Date();

        usuarioRol = (UsuarioRol) getIntent().getExtras().get("usuarioRol");

        if(usuarioRol.getRol().equals("centroEscolar")) {
            // TODO: Poner bien al terminar
            //keyCentro = usuarioRol.getUsuario().replace(".","");
            keyCentro = "scorazon@gmailcom";
            cargarValores(diaActual);
        }
        else{
            String keyUsuario = usuarioRol.getUsuario().replace(".","");

            // Recupero el centro al que pertenece el usuario
            myRef = database.getReference("usuarioCentro/" + keyUsuario + "/centro");
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    // TODO: Poner bien al terminar
                    keyCentro = "scorazon@gmailcom";
                    //keyCentro = (String) dataSnapshot.getValue();
                    cargarValores(diaActual);
                }
                @Override
                public void onCancelled(DatabaseError error) {
                    Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos.", error.toException());
                }
            });

        }
    }

    // Traemos a memoria los componentes gráficos que vamos a utilizar constantemente
    private void cargarElementosVista() {
        dia1= findViewById(R.id.semana_dia1_fecha);
        dia2= findViewById(R.id.semana_dia2_fecha);
        dia3= findViewById(R.id.semana_dia3_fecha);
        dia4= findViewById(R.id.semana_dia4_fecha);
        dia5= findViewById(R.id.semana_dia5_fecha);

        estadoDia1 = findViewById(R.id.semana_dia1_estado_texto);
        estadoDia1.setOnClickListener(this);
        estadoDia2 = findViewById(R.id.semana_dia2_estado_texto);
        estadoDia2.setOnClickListener(this);
        estadoDia3 = findViewById(R.id.semana_dia3_estado_texto);
        estadoDia3.setOnClickListener(this);
        estadoDia4 = findViewById(R.id.semana_dia4_estado_texto);
        estadoDia4.setOnClickListener(this);
        estadoDia5 = findViewById(R.id.semana_dia5_estado_texto);
        estadoDia5.setOnClickListener(this);

        estadoColor1 = findViewById(R.id.semana_dia1_estado_color);
        estadoColor2 = findViewById(R.id.semana_dia2_estado_color);
        estadoColor3 = findViewById(R.id.semana_dia3_estado_color);
        estadoColor4 = findViewById(R.id.semana_dia4_estado_color);
        estadoColor5 = findViewById(R.id.semana_dia5_estado_color);
    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        switch (i){
            case R.id.image_anterior:
                cargarDiasAnteriores();
                break;
            case R.id.image_siguiente:
                cargarDiasPosteriores();
                break;
            case R.id.semana_dia1_estado_texto:
                irDetalleDiaCalendario(dia1.getText().toString(), estadoDia1.getText().toString());
                break;
            case R.id.semana_dia2_estado_texto:
                irDetalleDiaCalendario(dia2.getText().toString(), estadoDia2.getText().toString());
                break;
            case R.id.semana_dia3_estado_texto:
                irDetalleDiaCalendario(dia3.getText().toString(), estadoDia3.getText().toString());
                break;
            case R.id.semana_dia4_estado_texto:
                irDetalleDiaCalendario(dia4.getText().toString(), estadoDia4.getText().toString());
                break;
            case R.id.semana_dia5_estado_texto:
                irDetalleDiaCalendario(dia5.getText().toString(), estadoDia5.getText().toString());
                break;
        }
    }

    // Cambiamos a la activity del detalle de un día del calendario
    private void irDetalleDiaCalendario(String fecha, String estado){
        if(estado.equals("CERRADO")){
            Toast.makeText(this, "El día seleccionado no se encuentra disponible.", Toast.LENGTH_LONG).show();
        }
        else {
            Intent intent = new Intent(this, DiaCalendarioActivity.class);
            intent.putExtra("fecha", fecha);
            intent.putExtra("estado", estado);
            intent.putExtra("keyCentro", keyCentro);
            intent.putExtra("usuarioRol", usuarioRol);
            startActivity(intent);
        }
    }

    // Cargamos los días siguientes
    private void cargarDiasPosteriores() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(ultimoDia);
        calendar.add(Calendar.DAY_OF_YEAR, 1);
        Date nuevoDia = calendar.getTime();
        cargarValores(nuevoDia);
    }

    // Cargamos los días anteriores
    private void cargarDiasAnteriores() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(primerDia);
        calendar.add(Calendar.DAY_OF_YEAR, -5);
        Date nuevoDia = calendar.getTime();
        cargarValores(nuevoDia);
    }


    // Cargamos los valores de los 5 días que nos salen en pantalla
    public void cargarValores(Date diaInicio){
        primerDia = diaInicio;
        Date diaSeleccionado = diaInicio;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(diaSeleccionado);

        int posicion = 1;
        do{
            ultimoDia = diaSeleccionado;

            String diaSeleccionadoTexto = dateFormat.format(diaSeleccionado);

            // Obtenemos la referencia
            myRef = database.getReference("menus/" + keyCentro + "/" + diaSeleccionadoTexto);
            final Date tempDiaSeleccionado = diaSeleccionado;
            final int tempPosicion = posicion;
            myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {
                    GenericTypeIndicator<HashMap<String,Menu>> t = new GenericTypeIndicator<HashMap<String,Menu>>() {};
                    HashMap<String,Menu> menus = dataSnapshot.getValue(t);

                    String estado;
                    if(menus != null){
                        estado = checkEstado(menus.size(), tempDiaSeleccionado);
                        inicializarVista(tempPosicion, dateFormat.format(tempDiaSeleccionado), getColorByEstado(estado), estado);
                    }
                    else{
                        inicializarVista(tempPosicion, dateFormat.format(tempDiaSeleccionado), getColorByEstado("CERRADO"), "CERRADO");
                    }
                }

                @Override
                public void onCancelled(DatabaseError error) {
                    Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos calendario.", error.toException());
                }
            });


            calendar.add(Calendar.DAY_OF_YEAR, 1);
            diaSeleccionado = calendar.getTime();
            posicion++;
        }while(posicion < 6);
    }

    // Establecemos la política de estado para cada uno de los días
    // Finalmente ha quedado como una propiedad calculada de cada uno de los días
    private String checkEstado(int menus, Date tempDiaSeleccionado) {

        String estado = "CERRADO";
        if(dateFormat.format(tempDiaSeleccionado).equals(dateFormat.format(diaActual)) && menus == 2){
            estado = "DISPONIBLE";
        }
        else if(tempDiaSeleccionado.after(diaActual) && menus == 2){
            estado = "VOTACIÓN";
        }
        else if(menus == 2){
            estado = "VALORACIÓN";
        }

        return estado;
    }

    // Asignamos los estados y cada uno de los colores correspondientes a cada uno de los días.
    public void inicializarVista(int posicion, String fechaTexto, int color, String estado){
        switch (posicion){
            case 1:
                dia1.setText(fechaTexto);
                estadoDia1.setText(estado);
                estadoColor1.setBackgroundColor(color);
                break;
            case 2:
                dia2.setText(fechaTexto);
                estadoDia2.setText(estado);
                estadoColor2.setBackgroundColor(color);
                break;
            case 3:
                dia3.setText(fechaTexto);
                estadoDia3.setText(estado);
                estadoColor3.setBackgroundColor(color);
                break;
            case 4:
                dia4.setText(fechaTexto);
                estadoDia4.setText(estado);
                estadoColor4.setBackgroundColor(color);
                break;
            case 5:
                dia5.setText(fechaTexto);
                estadoDia5.setText(estado);
                estadoColor5.setBackgroundColor(color);
                break;
            default:
                break;
        }
    }

    // Cogemos el color que corresponda al estado solicitado
    public int getColorByEstado(String estado){
        int color;
        switch (estado){
            case "DISPONIBLE":
                color = Color.YELLOW;
                break;
            case "VOTACIÓN":
                color = Color.CYAN;
                break;
            case "VALORACIÓN":
                color = Color.GREEN;
                break;
            default :
                color = Color.RED;
                break;
        }
        return color;
    }

}
