package ruben.uoc.comedorescolar.activities;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.HashMap;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.model.Menu;
import ruben.uoc.comedorescolar.model.UsuarioRol;

public class DiaCalendarioActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    private String fecha, estado, keyCentro;
    private UsuarioRol usuarioRol;
    private TextView votoMenu1, votoMenu2, votarMenu1, votarMenu2, valorarMenu1, valorarMenu2;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dia_calendario);

        fecha = (String) getIntent().getExtras().get("fecha");
        estado = (String) getIntent().getExtras().get("estado");
        keyCentro = (String) getIntent().getExtras().get("keyCentro");
        usuarioRol = (UsuarioRol) getIntent().getExtras().get("usuarioRol");

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();

        // Obtenemos la referencia
        myRef = database.getReference("menus/" + keyCentro + "/" + fecha);
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<HashMap<String,Menu>> t = new GenericTypeIndicator<HashMap<String,Menu>>() {};
                HashMap<String,Menu> menus = dataSnapshot.getValue(t);

                inicializarVista(menus);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos calendario.", error.toException());
            }
        });

    }

    // Inicializamos los valores de la vista
    private void inicializarVista(HashMap<String, Menu> menus) {

        Menu primerMenu = menus.get("Primer Menú");
        Menu segundoMenu = menus.get("Segundo Menú");

        votoMenu1 = findViewById(R.id.votos_dia_menu1);
        votoMenu2 = findViewById(R.id.votos_dia_menu2);

        TextView nombreMenu1 = findViewById(R.id.nombre_menu1_votacion);
        nombreMenu1.setText(primerMenu.getNombre());
        TextView nombreMenu2 = findViewById(R.id.nombre_menu2_votacion);
        nombreMenu2.setText(segundoMenu.getNombre());

        LinearLayout menu1 = findViewById(R.id.menu1_layout_votado);
        LinearLayout menu2 = findViewById(R.id.menu2_layout_votado);

        // Destacamos el menú con mayor número de votos
        if(primerMenu.getVotos() >= segundoMenu.getVotos()){
            menu1.setBackground(getResources().getDrawable(R.drawable.border_opciones_menu));
        }
        else{
            menu2.setBackground(getResources().getDrawable(R.drawable.border_opciones_menu));
        }

        // Dependiendo de los roles y el estado del día, permitimos unas acciones u otras
        if(usuarioRol.getRol().equals("alumno")) {
            if (estado.equals("VOTACIÓN")) {
                votarMenu1 = findViewById(R.id.boton_votar_menu1);
                votarMenu1.setVisibility(View.VISIBLE);
                votarMenu1.setOnClickListener(this);

                votarMenu2 = findViewById(R.id.boton_votar_menu2);
                votarMenu2.setVisibility(View.VISIBLE);
                votarMenu2.setOnClickListener(this);
            }
            else if(estado.equals("VALORACIÓN")) {
                valorarMenu1 = findViewById(R.id.boton_valorar_menu1);
                valorarMenu1.setOnClickListener(this);
                if(primerMenu.getVotos() >= segundoMenu.getVotos()){
                    valorarMenu1.setVisibility(View.VISIBLE);
                }

                valorarMenu2 = findViewById(R.id.boton_valorar_menu2);
                valorarMenu2.setOnClickListener(this);
                if(primerMenu.getVotos() < segundoMenu.getVotos()){
                    valorarMenu2.setVisibility(View.VISIBLE);
                }
            }
        }

        TextView titulo_fecha = findViewById(R.id.titulo_seleccionar_menu);
        titulo_fecha.setText("Menús del " + fecha);


        // Menú 1
        String fotoMenuB64 = primerMenu.getFoto();
        byte[] fotoPByteArray = Base64.decode(fotoMenuB64, Base64.DEFAULT);
        Bitmap fotoBitmap = BitmapFactory.decodeByteArray(fotoPByteArray, 0, fotoPByteArray.length);

        ImageView imagenPrimerMenu = findViewById(R.id.image_menu1_dia);
        imagenPrimerMenu.setImageBitmap(fotoBitmap);

        String ingredientes = "";
        for(String ingrediente: primerMenu.getIngredientes()){
            ingredientes = ingredientes + " " + ingrediente;
        }

        TextView ingredientesPrimerMenu = findViewById(R.id.ingredientes_dia_menu1);
        ingredientesPrimerMenu.setText(ingredientes);

        votoMenu1.setText(String.valueOf(primerMenu.getVotos()));


        // Menú 2
        fotoMenuB64 = segundoMenu.getFoto();
        fotoPByteArray = Base64.decode(fotoMenuB64, Base64.DEFAULT);
        fotoBitmap = BitmapFactory.decodeByteArray(fotoPByteArray, 0, fotoPByteArray.length);

        ImageView imagenSegundoMenu = findViewById(R.id.image_menu2_dia);
        imagenSegundoMenu.setImageBitmap(fotoBitmap);

        ingredientes = "";
        for(String ingrediente: segundoMenu.getIngredientes()){
            ingredientes = ingredientes + " " + ingrediente;
        }

        TextView ingredientesSegundoMenu = findViewById(R.id.ingredientes_dia_menu2);
        ingredientesSegundoMenu.setText(ingredientes);

        votoMenu2.setText(String.valueOf(segundoMenu.getVotos()));
    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        if(i == R.id.boton_votar_menu1){
            votar("Primer Menú");
        }
        else if(i == R.id.boton_votar_menu2){
            votar("Segundo Menú");
        }
        else if(i == R.id.boton_valorar_menu1){
            irValoracion("Primer Menú");
        }
        else if(i == R.id.boton_valorar_menu2){
            irValoracion("Segundo Menú");
        }
    }


    // Vamos a la pantalla de valoración
    private void irValoracion(String menu) {
        Intent intent = new Intent(this, ValoracionActivity.class);
        intent.putExtra("fecha", fecha);
        intent.putExtra("menu", menu);
        intent.putExtra("keyCentro", keyCentro);
        intent.putExtra("usuarioRol", usuarioRol);
        startActivity(intent);
    }


    // Iniciamos el proceso de votación
    private void votar(final String menu) {
        // Obtenemos la referencia
        myRef = database.getReference("votos/" + keyCentro + "/" + fecha + "/" + usuarioRol.getUsuario().replace(".","") );
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<Boolean> t = new GenericTypeIndicator<Boolean>() {};
                Boolean usuarioVota = dataSnapshot.getValue(t);

                if(usuarioVota != null){
                    Toast.makeText(DiaCalendarioActivity.this,"Usted ya ha votado.", Toast.LENGTH_LONG).show();
                }
                else{
                    myRef.setValue(true);
                    confirmarVoto(menu);
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos votos.", error.toException());
            }
        });

    }

    // Si los requisitos previos se han cumplido en el proceso de votación, confirmamos el voto y actualizamos valores.
    private void confirmarVoto(final String menu) {
        myRef = database.getReference("menus/" + keyCentro + "/" + fecha + "/" + menu + "/votos" );
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<Integer> t = new GenericTypeIndicator<Integer>() {};
                Integer votos = dataSnapshot.getValue(t);

                votos = votos + 1;
                myRef.setValue(votos);

                if(menu.equals("Primer Menú")){
                    votoMenu1.setText(String.valueOf(votos));
                }else{
                    votoMenu2.setText(String.valueOf(votos));
                }

                Toast.makeText(DiaCalendarioActivity.this,"Voto realizado.", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos votos.", error.toException());
            }
        });
    }


}
