package ruben.uoc.comedorescolar.activities;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.common.SignInButton;
import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.GoogleAuthProvider;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.model.UsuarioRol;

public class MainActivity extends AppCompatActivity implements View.OnClickListener{

    private static final String TAG_FIREBASE = "Firebase";
    private FirebaseAuth mAuth;
    private FirebaseUser user;

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    private static final String TAG_Google = "Google";
    private static final int RC_SIGN_IN = 9001;
    private GoogleSignInClient mGoogleSignInClient;
    private GoogleSignInAccount cuentaGoogle;

    private EditText email;
    private EditText password;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Instancia de Firebase.
        mAuth = FirebaseAuth.getInstance();

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();

        email = findViewById(R.id.email);
        password = findViewById(R.id.password);

        //Entrar con usuario normal.
        TextView botonLogin = findViewById(R.id.login);
        botonLogin.setOnClickListener(this);

        //Registrar usuario.
        TextView registro = findViewById(R.id.registro);
        registro.setOnClickListener(this);

        // Entrar con Google.
        SignInButton botonGoogle = findViewById(R.id.loginGoogle);
        botonGoogle.setColorScheme(SignInButton.COLOR_DARK);
        botonGoogle.setOnClickListener(this);

        TextView recordarPassword = findViewById(R.id.recordar_password);
        recordarPassword.setOnClickListener(this);

        // Configuración del login con Google
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(getString(R.string.default_web_client_id))
                .requestEmail()
                .build();

        mGoogleSignInClient = GoogleSignIn.getClient(this, gso);

    }

    @Override
    protected void onStart() {
        super.onStart();
        // Check if user is signed in (non-null) and update UI accordingly.
        user = mAuth.getCurrentUser();

        // Check for existing Google Sign In account, if the user is already signed in the GoogleSignInAccount will be non-null.
        cuentaGoogle = GoogleSignIn.getLastSignedInAccount(this);
    }

    // Lo llamamos a través del proceso de autenticación con Google.
    // Si la respuesta es satisfactoria, obtenemos la cuenta y procedemos a la autenticación.
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        // Result returned from launching the Intent from GoogleSignInClient.getSignInIntent(...);
        if (requestCode == RC_SIGN_IN) {
            // The Task returned from this call is always completed, no need to attach a listener.
            Task<GoogleSignInAccount> task = GoogleSignIn.getSignedInAccountFromIntent(data);
            try {
                cuentaGoogle = task.getResult(ApiException.class);
                loginAuthWithGoogle(cuentaGoogle);
            } catch (ApiException e) {
                // The ApiException status code indicates the detailed failure reason.
                Log.w(TAG_Google, "signInResult:failed code=" + e.getStatusCode());
            }
        }
    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        if (i == R.id.login) {
            loginUsuario(email.getText().toString(), password.getText().toString());
        } else if (i == R.id.registro) {
            crearCuenta(email.getText().toString(), password.getText().toString());
        } else if (i == R.id.loginGoogle) {
            // Iniciamos la actividad de Google para obtener la cuenta del usuario.
            Intent signInIntent = mGoogleSignInClient.getSignInIntent();
            startActivityForResult(signInIntent, RC_SIGN_IN);
        }else if(i == R.id.recordar_password){
            recuperarPasswordCorreo();
        }
    }


    // Login del usuario mediante usuario (correo) y contraseña.
    private void loginUsuario(String correo, String clave){
        mAuth.signInWithEmailAndPassword(correo, clave)
                .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            Log.d(TAG_FIREBASE, "LoginFirebase:success");
                            user = mAuth.getCurrentUser();
                            if(!user.isEmailVerified()){    // Comprobamos si la cuenta ha sido verificada por email.
                                Log.w(TAG_FIREBASE, "VerificarCuenta:failure", task.getException());
                                Toast.makeText(MainActivity.this, "La cuenta no ha sido verificada.", Toast.LENGTH_LONG).show();
                            }else{
                                checkRegistroCompletado();
                            }
                        } else {
                            Log.w(TAG_FIREBASE, "LoginFirebase:failure", task.getException());
                            Toast.makeText(MainActivity.this, "Error de autenticación.", Toast.LENGTH_LONG).show();
                        }
                    }
                });
    }

    // Login del usuario mediante su cuenta de Google.
    // Si la cuenta no la tenemos almacenada en nuestra aplicación, entonces se crea directamente.
    private void loginAuthWithGoogle(GoogleSignInAccount acct) {
        Log.d(TAG_FIREBASE, "firebaseAuthWithGoogle:" + acct.getId());
        AuthCredential credential = GoogleAuthProvider.getCredential(acct.getIdToken(), null);
        mAuth.signInWithCredential(credential).addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            Log.d(TAG_FIREBASE, "signInWithCredential:success");
                            user = mAuth.getCurrentUser();

                            checkRegistroCompletado(); // Comprobación de que existe en base de datos.
                        } else {
                            Log.w(TAG_FIREBASE, "signInWithCredential:failure", task.getException());
                            Toast.makeText(MainActivity.this, "Authentication failed.", Toast.LENGTH_SHORT).show();
                        }
                    }
                });
    }

    // Creamos una cuenta de usuario.
    // Para no tener datos obsoletos en base de datos, esperamos a que confirmen la cuenta creada y completen el registro.
    private void crearCuenta(String correo, String clave){
        if(correo == null || correo.isEmpty() || clave == null || clave.isEmpty()){
            Toast.makeText(this, "Por favor, complete los campos usuario y clave.", Toast.LENGTH_LONG).show();
        }
        else {
            mAuth.createUserWithEmailAndPassword(correo, clave)
                    .addOnCompleteListener(this, new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            if (task.isSuccessful()) {
                                // Sign in success, update UI with the signed-in user's information
                                Log.d(TAG_FIREBASE, "createUserWithEmail:success");
                                user = mAuth.getCurrentUser();
                                user.sendEmailVerification();

                                Toast.makeText(MainActivity.this, "Se ha enviado un correo de confirmación.", Toast.LENGTH_SHORT).show();

                            } else {
                                // If sign in fails, display a message to the user.
                                Log.w(TAG_FIREBASE, "createUserWithEmail:failure", task.getException());
                                Toast.makeText(MainActivity.this, "Authentication failed.", Toast.LENGTH_SHORT).show();
                            }
                        }
                    });
        }
    }

    // Comprobamos si el usuario ha completado el registro de nuestra aplicación
    // En este momento se encuentra registrado mediante usuario y contraseña o mediante la cuenta de Google,
    // por lo tanto, debe completar el resto de datos.
    private void checkRegistroCompletado(){
        // Obtenemos la referencia
        myRef = database.getReference("usuarioRol/" + user.getEmail().replace(".",""));

        // Se lee de base de datos (una sola vez)
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<UsuarioRol> t = new GenericTypeIndicator<UsuarioRol>(){};
                UsuarioRol usuarioActual = dataSnapshot.getValue(t);

                if(usuarioActual != null && user.getEmail().equals(usuarioActual.getUsuario()) ){
                    irPantallaMenu(usuarioActual);               // Si existe vamos al menú.
                }
                else{
                    usuarioActual = new UsuarioRol(user.getEmail(), null);
                    irPantallaRegistro(usuarioActual);           // No existe, vamos a nuestro propio registro.
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos.", error.toException());
            }
        });
    }

    // Se envía un correo para la recuperación de la contraseña.
    private void recuperarPasswordCorreo(){
        if(email.getText() != null){
            mAuth.sendPasswordResetEmail(email.getText().toString())
                    .addOnCompleteListener(new OnCompleteListener<Void>() {
                        @Override
                        public void onComplete(@NonNull Task<Void> task) {
                            if (task.isSuccessful()) {
                                Log.d(TAG_FIREBASE, "Email sent.");
                                Toast.makeText(MainActivity.this, "Se ha enviado un correo a su cuenta.", Toast.LENGTH_LONG).show();
                            }else{
                                Toast.makeText(MainActivity.this, "Error enviando corero para recordar contraseña.", Toast.LENGTH_LONG).show();
                            }
                        }
                    });
        } else {
            Toast.makeText(MainActivity.this, "Debe introducir su correo.", Toast.LENGTH_LONG).show();
        }
    }

    // Vamos a la pantalla de registro.
    private void irPantallaRegistro(UsuarioRol usuarioRol){
        Intent intentRegistro = new Intent(this, RegistroActivity.class);
        intentRegistro.putExtra("usuarioRol", usuarioRol);
        startActivity(intentRegistro);
    }

    // Vamos al menú.
    private void irPantallaMenu(UsuarioRol usuarioRol){
        Intent intentMenu = new Intent(this, MenuActivity.class);
        intentMenu.putExtra("usuarioRol", usuarioRol);
        startActivity(intentMenu);
    }


    // Desconectamos al usuario.
/*    private void desconectar() {
        // Firebase sign out
        mAuth.signOut();

        // Google sign out
        mGoogleSignInClient.signOut().addOnCompleteListener(this,
                new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        updateUI(null);
                    }
                });
    }*/

}
