package ruben.uoc.comedorescolar.activities;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.adapters.SpinnerAdapterCentro;
import ruben.uoc.comedorescolar.model.Alumno;
import ruben.uoc.comedorescolar.model.CentroEscolar;
import ruben.uoc.comedorescolar.model.UsuarioRol;


public class RegistroActivity extends AppCompatActivity implements View.OnClickListener{

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    private UsuarioRol usuarioRol;
    private Spinner spinnerCentros;

    private final int REQUEST_CODE_CAMERA = 1;
    private ImageView imagenAlumno;
    private TextView infoHacerFoto;
    private Bitmap fotoAlumno;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_registro);

        // Botón registrar como centro escolar
        TextView registroCentro = findViewById(R.id.ir_registro_centro);
        registroCentro.setOnClickListener(this);

        // Botón registrar como alumno
        TextView registroAlumno = findViewById(R.id.ir_registro_alumno);
        registroAlumno.setOnClickListener(this);

        // Botón confirmar registro centro escolar
        TextView confirmarRegistroCentro = findViewById(R.id.confirmar_registro_centro);
        confirmarRegistroCentro.setOnClickListener(this);

        // Botón confirmar registro alumno
        TextView confirmarRegistroAlumno = findViewById(R.id.confirmar_registro_alumno);
        confirmarRegistroAlumno.setOnClickListener(this);

        // Botón para hacer foto al alumno
        infoHacerFoto = findViewById(R.id.info_image_alumno);
        infoHacerFoto.setOnClickListener(this);
        imagenAlumno = findViewById(R.id.image_alumno);

        usuarioRol = (UsuarioRol) getIntent().getSerializableExtra("usuarioRol");

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();
    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        if (i == R.id.ir_registro_centro) {
            findViewById(R.id.contenedor_registro).setVisibility(View.INVISIBLE);
            findViewById(R.id.contenedor_registro_centro).setVisibility(View.VISIBLE);
        }
        else if(i == R.id.ir_registro_alumno){
            findViewById(R.id.contenedor_registro).setVisibility(View.INVISIBLE);
            findViewById(R.id.contenedor_registro_alumno).setVisibility(View.VISIBLE);

            inicializarSpinnerCentros();
        }
        else if(i == R.id.confirmar_registro_centro){
            confirmarRegistroCentro();
        }
        else if(i == R.id.confirmar_registro_alumno){
            confirmarRegistroAlumno();
        }
        else if(i == R.id.info_image_alumno){
            hacerFofo();
        }
    }

    // Recogemos los datos de la cámara
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == Activity.RESULT_OK && requestCode == REQUEST_CODE_CAMERA) {
            Bundle extras = data.getExtras();
            fotoAlumno = (Bitmap) extras.get("data");
            imagenAlumno.setImageBitmap(fotoAlumno);
            imagenAlumno.setVisibility(View.VISIBLE);
            infoHacerFoto.setVisibility(View.GONE);
        }
    }


    // Confirmamos el registro, si todos los datos están correctos se crea correctamente el centro
    private void confirmarRegistroCentro() {
        usuarioRol.setRol("centroEscolar");

        String nombreCentro = ((EditText)findViewById(R.id.nombre_centro)).getText().toString();
        String cip = ((EditText)findViewById(R.id.cip)).getText().toString();

        if(nombreCentro.isEmpty() || cip.isEmpty()){
            Toast.makeText(this, "Debe completar todos los datos.", Toast.LENGTH_LONG).show();
        }
        else{
            CentroEscolar centroEscolar = new CentroEscolar(nombreCentro, cip, usuarioRol.getUsuario());
            final String keyCentro = centroEscolar.getCorreo().replace(".","");

            myRef = database.getReference("centrosEscolares");
            myRef.child(keyCentro).setValue(centroEscolar).addOnCompleteListener(new OnCompleteListener<Void>() {
                @Override
                public void onComplete(@NonNull Task<Void> task) {
                    if(task.isSuccessful()){
                        myRef = database.getReference("usuarioRol");
                        myRef.child(keyCentro).setValue(usuarioRol);

                        Toast.makeText(RegistroActivity.this, "Centro escolar registrado.", Toast.LENGTH_LONG).show();

                        irPantallaMenu(usuarioRol);
                    }
                    else{
                        Toast.makeText(RegistroActivity.this, "Error al confirmar registro.", Toast.LENGTH_LONG).show();
                    }
                }
            });
        }
    }

    // Confirmamos el registro, si todos los datos están correctos se crea correctamente el alumno
    private void confirmarRegistroAlumno() {
        final Alumno nuevoAlumno = validarDatos();
        usuarioRol.setRol("alumno");

        Spinner spinner = findViewById(R.id.spinner_centros_escolares);
        CentroEscolar centro = (CentroEscolar) spinner.getSelectedItem();

        if(nuevoAlumno == null){
            Toast.makeText(RegistroActivity.this, "Debe completar todos los datos.", Toast.LENGTH_LONG).show();
        }
        else {
            final String keyCentro = centro.getCorreo().replace(".","");
            final String keyAlumno = nuevoAlumno.getCorreoResponsable().replace(".","");

            myRef = database.getReference("alumnos");
            myRef.child(keyCentro).child(keyAlumno).setValue(nuevoAlumno).addOnCompleteListener(new OnCompleteListener<Void>() {
                @Override
                public void onComplete(@NonNull Task<Void> task) {
                    if(task.isSuccessful()){
                        myRef = database.getReference("usuarioRol");
                        myRef.child(keyAlumno).setValue(usuarioRol);

                        myRef = database.getReference("usuarioCentro");
                        myRef.child(keyAlumno).child("centro").setValue(keyCentro);

                        Toast.makeText(RegistroActivity.this, "Alumno registrado.", Toast.LENGTH_LONG).show();

                        irPantallaMenu(usuarioRol);
                    }
                    else{
                        Toast.makeText(RegistroActivity.this, "Error al confirmar registro.", Toast.LENGTH_LONG).show();
                    }
                }
            });
        }
    }

    // Validación de los datos introducidos del alumno
    private Alumno validarDatos() {
        String dniResponsable = ((EditText)findViewById(R.id.dni_responsable)).getText().toString();
        String dniAlumno = ((EditText)findViewById(R.id.dni_alumno)).getText().toString();
        String nombreAlumno = ((EditText)findViewById(R.id.nombre_alumno)).getText().toString();
        String apellido1Alumno = ((EditText)findViewById(R.id.apellido1_alumno)).getText().toString();
        String apellido2Alumno = ((EditText)findViewById(R.id.apellido2_alumno)).getText().toString();

        if(fotoAlumno == null || dniResponsable.isEmpty() || dniAlumno.isEmpty() || nombreAlumno.isEmpty()
                || apellido1Alumno.isEmpty() || apellido2Alumno.isEmpty()){
            return null;
        }

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        fotoAlumno.compress(Bitmap.CompressFormat.PNG, 100, byteArrayOutputStream);
        byte[] fotoAlumnoByteArray = byteArrayOutputStream.toByteArray();
        String fotoAlumnoEncodedB64 = Base64.encodeToString(fotoAlumnoByteArray, Base64.DEFAULT);

        RadioGroup grupoRadioButtons = findViewById(R.id.grupo_radioButton_parentesco);
        int selectedId = grupoRadioButtons.getCheckedRadioButtonId();
        String parentesco = ((RadioButton) findViewById(selectedId)).getText().toString();

        Alumno nuevoAlumno = new Alumno(usuarioRol.getUsuario(), dniResponsable, parentesco, dniAlumno,
                nombreAlumno, apellido1Alumno, apellido2Alumno, fotoAlumnoEncodedB64);

        return nuevoAlumno;
    }

    // Vamos al menú.
    private void irPantallaMenu(UsuarioRol usuarioRol){
        Intent intentMenu = new Intent(this, MenuActivity.class);
        intentMenu.putExtra("usuarioRol", usuarioRol);
        startActivity(intentMenu);
    }

    // Cargamos los valores del spinner con los centros escolares que tenemos en base de datos
    private void inicializarSpinnerCentros() {
        // Obtenemos la referencia
        myRef = database.getReference("centrosEscolares");

        // Se lee de base de datos (una sola vez)
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<HashMap<String, CentroEscolar>> t = new GenericTypeIndicator<HashMap<String, CentroEscolar>>(){};
                HashMap<String,CentroEscolar> centros = dataSnapshot.getValue(t);

                ArrayList<CentroEscolar> centrosEscolares = new ArrayList<>(centros.values());

                spinnerCentros = findViewById(R.id.spinner_centros_escolares);
                SpinnerAdapterCentro arrayAdapter = new SpinnerAdapterCentro(RegistroActivity.this, android.R.layout.simple_spinner_item, centrosEscolares);
                arrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_item);
                spinnerCentros.setAdapter(arrayAdapter);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos.", error.toException());
            }
        });

    }

    // Activamos la cámara para hacer la foto correspondiente
    private void hacerFofo() {
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        if (takePictureIntent.resolveActivity(getPackageManager()) != null) {
            startActivityForResult(takePictureIntent, REQUEST_CODE_CAMERA);
        } else {
            Toast.makeText(this, "La cámara no está disponible.", Toast.LENGTH_LONG).show();
        }
    }


}
