package ruben.uoc.comedorescolar.activities;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.util.Base64;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.model.Menu;

public class RegistroMenuActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    // Código de petición para seleccionar la imagen de nuestro dispositivo
    private final int REQUEST_IMAGE = 1;

    private ImageView imagenMenu;
    private TextView infoSeleccionarFoto;
    private Bitmap fotoMenu;

    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_registro_menu);

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();

        // Confirmar registro
        TextView confirmarRegistro = findViewById(R.id.confirmar_registro_menu);
        confirmarRegistro.setOnClickListener(this);

        // Botón para hacer foto al alumno
        infoSeleccionarFoto = findViewById(R.id.info_image_menu);
        infoSeleccionarFoto.setOnClickListener(this);
        imagenMenu = findViewById(R.id.image_menu);
    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        if (i == R.id.confirmar_registro_menu) {
            confirmarRegistro();
        }
        else if(i == R.id.info_image_menu){
            seleccionarFoto();
        }

    }

    // Recuperamos los datos de la foto seleccionada
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent resultData) {
        if (requestCode == REQUEST_IMAGE && resultCode == Activity.RESULT_OK) {
            if (resultData != null) {
                try{
                    Uri imageUri = resultData.getData();
                    InputStream imageStream = getContentResolver().openInputStream(imageUri);
                    fotoMenu = BitmapFactory.decodeStream(imageStream);
                    imagenMenu.setImageBitmap(fotoMenu);
                    imagenMenu.setVisibility(View.VISIBLE);
                    infoSeleccionarFoto.setVisibility(View.GONE);
                } catch (FileNotFoundException e) {
                    Toast.makeText(this, "Error cargando la imagen.", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

    // Iniciamos el proceso de seleccionar foto, mediante los recursos del dispositivo como la galería de fotos, entre otros.
    private void seleccionarFoto(){
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("image/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, REQUEST_IMAGE);
    }


    // Confirmamos el registro, añadiendo un nuevo menú al día elegido.
    private void confirmarRegistro(){
        final Menu nuevoMenu = validarDatos();

        RadioGroup grupoRadioButtons = findViewById(R.id.grupo_radioButton_menu);
        int selectedId = grupoRadioButtons.getCheckedRadioButtonId();
        String numMenu = ((RadioButton) findViewById(selectedId)).getText().toString();

        if(nuevoMenu == null){
            Toast.makeText(RegistroMenuActivity.this, "Debe completar todos los datos.", Toast.LENGTH_LONG).show();
        }
        else {
            String centro = (String) getIntent().getExtras().get("usuario");
            //final String keyCentro = centro.replace(".","");
            // TODO: Poner bien al terminar
            final String keyCentro = "scorazon@gmailcom";

            myRef = database.getReference("menus");
            myRef.child(keyCentro).child(nuevoMenu.getFechaPublicacion()).child(numMenu).setValue(nuevoMenu).addOnCompleteListener(this, new OnCompleteListener<Void>() {
                @Override
                public void onComplete(@NonNull Task<Void> task) {
                    if(task.isSuccessful()){
                        Toast.makeText(RegistroMenuActivity.this, "Menu registrado correctamente.", Toast.LENGTH_LONG).show();
                    }
                    else{
                        Toast.makeText(RegistroMenuActivity.this, "Error al confirmar registro.", Toast.LENGTH_LONG).show();
                    }
                }
            });
        }
    }

    // Validamos que los datos sean correctos
    private Menu validarDatos() {
        String nombre = ((EditText)findViewById(R.id.nombre_menu)).getText().toString();
        String fecha = ((EditText)findViewById(R.id.fecha_menu)).getText().toString();
        String ingrediente1 = ((EditText)findViewById(R.id.ingrediente1_menu)).getText().toString();
        String ingrediente2 = ((EditText)findViewById(R.id.ingrediente2_menu)).getText().toString();
        String ingrediente3 = ((EditText)findViewById(R.id.ingrediente3_menu)).getText().toString();
        String ingrediente4 = ((EditText)findViewById(R.id.ingrediente4_menu)).getText().toString();
        String ingrediente5 = ((EditText)findViewById(R.id.ingrediente5_menu)).getText().toString();

        // Comprobamos que es una fecha
        try {
            dateFormat.parse(fecha);
        } catch (ParseException e) {
            return null;
        }

        // Comprobamos que no hay campos vacíos
        // Consideramos que solo es obligatorio introducir un ingrediente
        if(fotoMenu == null || nombre.isEmpty() || ingrediente1.isEmpty()){
            return null;
        }

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        fotoMenu.compress(Bitmap.CompressFormat.PNG, 100, byteArrayOutputStream);
        byte[] fotoMenuByteArray = byteArrayOutputStream.toByteArray();
        String fotoMenuEncodedB64 = Base64.encodeToString(fotoMenuByteArray, Base64.DEFAULT);

        ArrayList<String> ingredientes = new ArrayList<>();
        ingredientes.add(ingrediente1);
        ingredientes.add(ingrediente2);
        ingredientes.add(ingrediente3);
        ingredientes.add(ingrediente4);
        ingredientes.add(ingrediente5);

        Menu nuevoMenu = new Menu(nombre, fecha, fotoMenuEncodedB64, ingredientes);

        return nuevoMenu;
    }
}
