package ruben.uoc.comedorescolar.activities;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.RatingBar;
import android.widget.TextView;
import android.widget.Toast;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.HashMap;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.model.Menu;
import ruben.uoc.comedorescolar.model.UsuarioRol;

public class ValoracionActivity extends AppCompatActivity implements View.OnClickListener {

    private static final String TAG_FIREBASE_DATABASE = "Firebase Database";
    private FirebaseDatabase database;
    private DatabaseReference myRef;

    private String fecha, numeroMenu, keyCentro;
    private UsuarioRol usuarioRol;
    private TextView media;
    private int numValoraciones;
    private RatingBar ratingBar;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_valoracion);

        // Instancia de BBDD Firebase.
        database = FirebaseDatabase.getInstance();

        fecha = (String) getIntent().getExtras().get("fecha");
        numeroMenu = (String) getIntent().getExtras().get("menu");
        keyCentro = (String) getIntent().getExtras().get("keyCentro");
        usuarioRol = (UsuarioRol) getIntent().getExtras().get("usuarioRol");


        // Obtenemos la referencia
        myRef = database.getReference("menus/" + keyCentro + "/" + fecha + "/" + numeroMenu);
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<Menu> t = new GenericTypeIndicator<Menu>() {};
                final Menu menu = dataSnapshot.getValue(t);

                myRef = database.getReference("puntuaciones/" + keyCentro + "/" + fecha);
                myRef.addListenerForSingleValueEvent(new ValueEventListener() {
                    @Override
                    public void onDataChange(DataSnapshot dataSnapshot) {
                        GenericTypeIndicator<HashMap<String,Boolean>> t = new GenericTypeIndicator<HashMap<String, Boolean>>() {};
                        HashMap<String,Boolean> valoraciones = dataSnapshot.getValue(t);
                        numValoraciones = (valoraciones == null) ? 0 : valoraciones.size();
                        inicializarVista(menu);
                    }

                    @Override
                    public void onCancelled(DatabaseError error) {
                        Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos calendario.", error.toException());
                    }
                });
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos calendario.", error.toException());
            }
        });

    }

    // Inicializamos los valores de la vista
    private void inicializarVista(Menu menu) {

        TextView nombreMenu = findViewById(R.id.nombre_menu_valoracion);
        nombreMenu.setText(menu.getNombre());

        String fotoMenuB64 = menu.getFoto();
        byte[] fotoPByteArray = Base64.decode(fotoMenuB64, Base64.DEFAULT);
        Bitmap fotoBitmap = BitmapFactory.decodeByteArray(fotoPByteArray, 0, fotoPByteArray.length);

        ImageView imagenPrimerMenu = findViewById(R.id.image_menu_valoracion);
        imagenPrimerMenu.setImageBitmap(fotoBitmap);

        TextView votos = findViewById(R.id.votos_menu_valoracion);
        votos.setText(String.valueOf(menu.getVotos()));

        // Calculamos la puntuación media, propiedad calculada
        // Guardamos la suma total de todas las puntuaciones y el número de votantes
        media = findViewById(R.id.media_menu_valoracion);
        float puntuacionMedia = (numValoraciones == 0) ? 0 : (menu.getPuntuacion()/numValoraciones);
        media.setText(String.valueOf(puntuacionMedia));

        ratingBar = findViewById(R.id.ratingBar_menu_valoracion);
        ratingBar.setRating(puntuacionMedia);

        TextView enviarValoracion = findViewById(R.id.button_enviar_valoracion);
        enviarValoracion.setOnClickListener(this);

    }

    @Override
    public void onClick(View view) {
        int i = view.getId();
        if(i == R.id.button_enviar_valoracion){
            valorar();
        }
    }

    // Iniciamos el proceso de valoración
    private void valorar() {
        // Obtenemos la referencia
        myRef = database.getReference("puntuaciones/" + keyCentro + "/" + fecha + "/" + usuarioRol.getUsuario().replace(".","") );
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<Boolean> t = new GenericTypeIndicator<Boolean>() {};
                Boolean usuarioValora = dataSnapshot.getValue(t);

                if(usuarioValora != null){
                    Toast.makeText(ValoracionActivity.this,"Usted ya ha enviado su valoración.", Toast.LENGTH_LONG).show();
                }
                else{
                    myRef.setValue(true);
                    confirmarValoracion();
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos votos.", error.toException());
            }
        });
    }

    // Si los requisitos previos se han cumplido en el proceso de valoración, finalmente confirmamos la valoración y actualizamos los valores
    private void confirmarValoracion() {
        myRef = database.getReference("menus/" + keyCentro + "/" + fecha + "/" + numeroMenu + "/puntuacion" );
        myRef.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<Double> t = new GenericTypeIndicator<Double>() {};
                Double puntuacion = dataSnapshot.getValue(t);

                puntuacion = puntuacion + ratingBar.getRating();
                myRef.setValue(puntuacion);

                numValoraciones = numValoraciones + 1;

                media.setText(String.valueOf(puntuacion/(numValoraciones)));

                Toast.makeText(ValoracionActivity.this,"Valoración enviada.", Toast.LENGTH_LONG).show();
            }

            @Override
            public void onCancelled(DatabaseError error) {
                Log.w(TAG_FIREBASE_DATABASE, "Error de lectura de base de datos votos.", error.toException());
            }
        });
    }
}
