package ruben.uoc.comedorescolar.adapters;


import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.support.v7.widget.CardView;
import android.support.v7.widget.RecyclerView;
import android.util.Base64;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.List;

import ruben.uoc.comedorescolar.R;
import ruben.uoc.comedorescolar.activities.AlumnoPerfilActivity;
import ruben.uoc.comedorescolar.model.Alumno;

public class RecyclerViewAdapterAlumno extends  RecyclerView.Adapter<RecyclerViewAdapterAlumno.AlumnoViewHolder> {

    // Clase interna que asociamos a nuestro adaptador de tipo RecyclerView.ViewHolder
    public static class AlumnoViewHolder extends RecyclerView.ViewHolder {

        public Alumno alumnoSeleccionado;
        public final CardView cardView;
        public final TextView nombreAlumno;
        public final ImageView fotoAlumno;

        public AlumnoViewHolder(final View itemView) {
            super(itemView);
            cardView = itemView.findViewById(R.id.cardView_alumno);
            nombreAlumno = itemView.findViewById(R.id.listado_nombre_alumno);
            fotoAlumno = itemView.findViewById(R.id.listado_foto_alumno);
        }
    }

    // Constantes para diferencia entre un elemento de la lista par o impar.
    private final int EVEN = 0;
    private final int ODD = 1;
    private List<Alumno> alumnos;


    public RecyclerViewAdapterAlumno(List<Alumno> listaAlumnos) {
        this.alumnos = listaAlumnos;
    }

    //Asociamos un tipo de CardView distinto si es par o impar
    @Override
    public AlumnoViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View v;
        switch (viewType) {
            case EVEN:
                v =  LayoutInflater.from(parent.getContext()).inflate(R.layout.card_view_alumno_par, parent, false);
                break;
            default:
                v =  LayoutInflater.from(parent.getContext()).inflate(R.layout.card_view_alumno_impar, parent, false);
                break;
        }

        return new AlumnoViewHolder(v);
    }

    // Rellenamos los dados de nuestro ViewHolder, seleccionando el alumno correspondiente de la lista
    @Override
    public void onBindViewHolder(final AlumnoViewHolder holder, int position) {

        final int currentPos = holder.getAdapterPosition();

        final Alumno alumnoActual = alumnos.get(currentPos);
        holder.alumnoSeleccionado = alumnoActual;
        String nombreCompleto = alumnoActual.getNombreAlumno() + " " + alumnoActual.getApellido1Alumno() + " " + alumnoActual.getApellido2Alumno();
        holder.nombreAlumno.setText(nombreCompleto);

        // Aquí asociamos la foto del alumno. Para ello ahora hay que volver a transformarla.
        String fotoB64 = alumnoActual.getFotoAlumnoEncodedB64();
        byte[] fotoAlumnoByteArray = Base64.decode(fotoB64, Base64.DEFAULT);
        Bitmap fotoAlumnoBitmap = BitmapFactory.decodeByteArray(fotoAlumnoByteArray, 0, fotoAlumnoByteArray.length);
        holder.fotoAlumno.setImageBitmap(fotoAlumnoBitmap);

        // Se dispara cuando hacemos click en un alumno
        holder.cardView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Context context = v.getContext();
                Intent intent = new Intent(context, AlumnoPerfilActivity.class);
                intent.putExtra("usuario", alumnoActual.getCorreoResponsable());
                context.startActivity(intent);
            }
        });
    }

    // Devuelve el número de alumnos.
    @Override
    public int getItemCount() {
        return alumnos.size();
    }

    // Comprobamos si es par o impar, de modo que, podamos asignar un layout u otro dependiendo del tipo.
    @Override
    public int getItemViewType(int position) {
        if (position % 2 == 0) {
            return EVEN;
        } else {
            return ODD;
        }
    }

    // Enganchamos nuestro RecyclerView.
    @Override
    public void onAttachedToRecyclerView(RecyclerView recyclerView) {
        super.onAttachedToRecyclerView(recyclerView);
    }
}
